/* 
 * The contents of this file are subject to the GrammarSoft Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.grammarsoft.com/GSPL
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the VISL CG Parser.
 * 
 * The Initial Developer of the Original Code is Martin Carlsen, 
 * GrammarSoft, and SDU Odense University.  
 * Portions created by Martin Carlsen, Grammarsoft, and SDU Odense 
 * University are Copyright (C) 1999-2000 Martin Carlsen, GrammarSoft, 
 * and SDU Odense University.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the GSPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the GSPL or the
 * GPL.
 */
#include "Debug.hh"
#include "Options.hh"

Debug * Debug::_instance = 0;
Debug * Debug::instance()
{
  if (_instance == 0)
    _instance = new Debug();
  return _instance;
}
Debug * Debug::init(Options * options)
{
  Debug * debug = Debug::instance();
  
  // debug/trace options:
  // --reductions
  debug->reductions = options->getSetting("reductions","no") == "";
  // --trace
  debug->trace_mappings = options->getSetting("trace","no") == "";
  debug->trace_constraints = options->getSetting("trace","no") == "";
  if(!debug->trace_mappings)
    {
      // --trace-mappings
      debug->trace_mappings = options->getSetting("trace-mappings","no") == "";
      // --trace-constraints
      debug->trace_constraints = options->getSetting("trace-constraints","no") == "";
    }
  // --verbosity
  debug->verbose = options->getSetting("verbose","no") == "" ||
    options->getSetting("verbosity","no") == "verbose";
  debug->minimal = options->getSetting("minimal","no") == "" ||
    options->getSetting("verbosity","no") == "minimal";
  debug->quiet = options->getSetting("quiet","no") == "" ||
    options->getSetting("verbosity","no") == "quiet";
  
  // --check-only
  debug->check_only = options->getSetting("check-only","no") == "";
  
  // --debug and --visldebug
  debug->debug = options->getSetting("debug","no") == "" || 
    options->getSetting("visldebug","no") == "";
  // --debug
  debug->debug_cg2 = options->getSetting("debug","no") == "" ;
  
  return debug;
}
