/* 
 * The contents of this file are subject to the GrammarSoft Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.grammarsoft.com/GSPL
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the VISL CG Parser.
 * 
 * The Initial Developer of the Original Code is Martin Carlsen, 
 * GrammarSoft, and SDU Odense University.  
 * Portions created by Martin Carlsen, Grammarsoft, and SDU Odense 
 * University are Copyright (C) 1999-2000 Martin Carlsen, GrammarSoft, 
 * and SDU Odense University.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the GSPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the GSPL or the
 * GPL.
 */
#ifndef _Debug_hh_
#define _Debug_hh_

#include "Options.hh"

#ifdef nodebugging

#define rout if(false) cout
#define tmout if(false) cout
#define tcout if(false) cout
#define dout(A) if(false) cout
#define derr(A) if(false) cerr
#define qerr if(false) cerr
#define qout if(false) cout

#else

#define rout if(Debug::instance()->reductions && !Debug::instance()->quiet) cout
#define tmout if(Debug::instance()->trace_mappings && !Debug::instance()->quiet) cout
#define tcout if(Debug::instance()->trace_constraints && !Debug::instance()->quiet) cout
#define derr(A) if((A) && !Debug::instance()->quiet) cerr
#define dout(A) if((A) && !Debug::instance()->quiet) cout
#define qerr if(!Debug::instance()->quiet) cerr
#define qout if(!Debug::instance()->quiet) cout

#endif

class Debug
{
private:
  static Debug * _instance;
  Debug() 
    {
    }
public:
  static Debug * init(Options * opts);
  static Debug * instance();
  
  bool reductions; // output all reductions while reading rules.
  bool trace_mappings; // trace all applied mappings.
  bool trace_constraints; // trace all applied constraints.
  bool minimal; // minimal trace output.
  bool verbose; // verbose trace output.
  bool quiet; // never output any messages.
  bool check_only; // do not parse any input, just check the syntax of the rules.
  bool debug; // warn when removing <Correct!> tags.
  bool debug_cg2; // Debug in cg2 compatible mode.
};

#endif
