%option c++
%option prefix="cgrules"
%option noyywrap
%x comment
%{
/* 
 * The contents of this file are subject to the GrammarSoft Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.grammarsoft.com/GSPL
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the VISL CG Parser.
 * 
 * The Initial Developer of the Original Code is Martin Carlsen, 
 * GrammarSoft, and SDU Odense University.  
 * Portions created by Martin Carlsen, Grammarsoft, and SDU Odense 
 * University are Copyright (C) 1999-2000 Martin Carlsen, GrammarSoft, 
 * and SDU Odense University.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the GSPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the GSPL or the
 * GPL.
 */
#include <string>
#include <iostream>
#include "cg.data_structures.hh"
#include "parser.tab.h"
#include "stdlib.h"
int numlines = 1; 
int balance = 0;
int context = 0;
Position * parseposition(char*);
/*
id		({letter}|{ad})|(({letter}|{special}|{semi})+({letter}|{digit}|{plusminus}|{special})+)
<comment>[\n]	{++numlines; BEGIN(INITIAL);}
*/
%}

line            .*\n
delim		[ \t]
eol		\n
ws		{delim}+
letter		[a-zA-Z]
digit		[0-9]
digits          {digit}+
special		[\"\'\!\?\.\,\:\&\$\<\>]
char1           [^ \t\n\#\;\=\(\)\|]
char            [^ \t\n\(\)\\\;]
qchar		[^ \t\n\\\"]
qoute		[\"]
escaped         \\.
id              ({qoute}({qchar}|{escaped})*{qoute})|({char1}({char}|{escaped})*)
position        (\*\*?)?\@?(\+|\-)?{digits}[cC]?
%%
"#"		BEGIN(comment);
<comment>{line}	{ BEGIN(INITIAL); ++numlines; }
{ws}		{}
{eol}		{++numlines;}
delimiters	{return(delimiters);}
"preferred-targets" {return(preferred);} 
constraints     {return(CONSTRAINTS);}
end		{return(END);}
remove		{return(REMOVE);}
select		{return(SELECT);}
list		{return(LIST);}
sets		{return(SETS);}
corrections     {return(CORRECTIONS);}
mappings	{return(MAPPINGS);}
set		{return(SET);}
substitute      {return(SUBSTITUTE);}
append          {return(APPEND);}
add		{return(ADD);}
map		{return(MAP);}
replace		{return(REPLACE);}
and		{return(AND);}
or		{return(OR);}
not		{return(NOT);}
link		{return(LINK);}
barrier         {return(BARRIER);}
if		{return(IF);}
target		{return(TARGET);}
{position}      {yylval.position = parseposition(yytext); return(POSITION);}
"+"/" "		{return('+');}
"-"/" "		{return('-');}
{id}		{yylval.tag = new string(yytext); string::size_type i; while ((i=yylval.tag->find("\\"))!=string::npos) yylval.tag->replace(i,1,""); return(ID);}
"="		{return(ASSIGNOP);}
";"		{return(';');}
"("		{++balance; return('(');}
")"		{--balance; return(')');}
"|"		{return(OR);}
%%

Position * parseposition( char * yytext )
{
  int i = -1;
  
  Scantype scan = scan_none;
  bool absolute = false;
  bool negative = false;
  int offset = 0;
  bool careful = false;
  while( char c = yytext[++i] )
    {
      switch(c)
      {
      case '*':
        if(scan==scan_asterisk) scan = scan_doubleasterisk;
	if(scan==scan_none) scan = scan_asterisk;
	break;
      case '@':
        absolute = true;
        break;
      case '-':
        negative = true;
      case '+':
        break;
      case 'C':
      case 'c':
        careful = true;
	break;
      default:
        offset = offset*10 + c - '0';
      }
    }
//    cout << (scan>scan_none?"*":"") << (scan>scan_asterisk?"*":"") << (absolute?"@":"") << (negative?"-":"") << offset << (careful?"C":"") << "\n";
    return new Position(scan, absolute, negative?-offset:offset, careful);
}
